"use strict";
const graphql = require("graphql");
const graphql_server_core_1 = require("graphql-server-core");
const rokot_mq_rabbit_1 = require("rokot-mq-rabbit");
const childLogger_1 = require("./childLogger");
class AMQPSubscription {
    constructor(options) {
        this.GRAPHQL_QUEUENAME = "graphql";
        if (!options) {
            throw new Error('Apollo Server requires options.');
        }
        if (arguments.length > 1) {
            throw new Error(`Apollo Server expects exactly one argument, got ${arguments.length}`);
        }
        const config = options.config || { host: "127.0.0.1", port: 5672 };
        const { logger } = options;
        this.logger = childLogger_1.createChildLogger(logger, 'AmqpSubscriptionServer');
        const factory = new rokot_mq_rabbit_1.RabbitMqConnectionFactory(logger, config);
        this.listener = new rokot_mq_rabbit_1.RabbitMqConsumer(logger, factory);
        this.graphqlOptions = options;
    }
    listenToQueries(cb) {
        return new Promise((resolve, reject) => this.listener.subscribe(this.GRAPHQL_QUEUENAME, msg => this.onMessage(msg).then(m => cb(m)))
            .then(disposer => {
            this.unsubscribeChannel = disposer;
            return resolve();
        }).catch(err => {
            this.logger.error(err, "failed to recieve message from queue '%s'", this.GRAPHQL_QUEUENAME);
            reject();
        }));
    }
    unsubscribe() {
        this.unsubscribeChannel().then(() => {
            this.logger.trace("cancelled channel from subscribing to queue '%s'", this.GRAPHQL_QUEUENAME);
        }).catch(err => {
            this.logger.error(err, "channel cancellation failed from queue '%j'", this.GRAPHQL_QUEUENAME);
        });
    }
    onMessage(queryOptions) {
        this.logger.trace("message received to process is '(%j)'", queryOptions);
        let isBatch = false;
        let context = this.graphqlOptions.context;
        if (isBatch) {
            context = Object.assign({}, context || {});
        }
        const formatErrorFn = this.graphqlOptions.formatError || graphql.formatError;
        let params = {
            schema: this.graphqlOptions.schema,
            query: queryOptions.query,
            variables: queryOptions.variables,
            context: context,
            rootValue: this.graphqlOptions.rootValue,
            operationName: queryOptions.operationName,
            logFunction: this.graphqlOptions.logFunction,
            validationRules: this.graphqlOptions.validationRules,
            formatError: formatErrorFn,
            formatResponse: this.graphqlOptions.formatResponse,
            debug: this.graphqlOptions.debug,
        };
        if (this.graphqlOptions.formatParams) {
            params = this.graphqlOptions.formatParams(params);
        }
        return graphql_server_core_1.runQuery(params);
    }
}
exports.AMQPSubscription = AMQPSubscription;
//# sourceMappingURL=rabbitmqApollo.js.map